@@ -1,5 +1,5 @@
 /*
- *      Copyright (C) 1997 Claus-Justus Heine
+ *      Copyright (C) 1997-2000 Claus-Justus Heine
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
@@ -22,8 +22,8 @@
  */
 
 char src[] = "$RCSfile: ftformat.c,v $";
-char rev[] = "$Revision: 1.9 $";
-char dat[] = "$Date: 1999/03/17 11:25:42 $";
+char rev[] = "$Revision: 1.11 $";
+char dat[] = "$Date: 2000/07/23 13:56:45 $";
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -45,14 +45,21 @@ char dat[] = "$Date: 1999/03/17 11:25:42 $";
 #include "ftfmt-bsm.h"
 #include "ftfmt-tapelib.h"
 
+extern void *xmalloc(size_t);
+extern void *xrealloc(void *, size_t);
+
+static struct opt_parms opts;
+static ftfmt_tpparms_t tpparms;
+
 int main(int argc, char *argv[])
 {
-    struct opt_parms opts;
-    ftfmt_tpparms_t tpparms;
     int tape_fd = -1;
     void *dma_buffer = NULL;
-    u_int8_t hseg[FT_SEGMENT_SIZE];
+    u_int8_t *hseg;
 
+    /* don't allocate on stack to catch buffer overflows more easily
+     */
+    hseg = xmalloc(FT_SEGMENT_SIZE);
     setlocale (LC_ALL, "");
     bindtextdomain (PACKAGE, LOCALEDIR);
     textdomain (PACKAGE);
@@ -103,6 +110,11 @@ int main(int argc, char *argv[])
     } else {
 	memset(hseg, 0, FT_SEGMENT_SIZE);
 	hseg[FT_FMT_ERROR] = 0xff; /* flag as invalid */
+	/* initialize some missing fields to sane values */
+	tpparms.frst_header_segment = 0;
+	tpparms.scnd_header_segment = 1;
+	tpparms.frst_data_segment = 2;
+	tpparms.last_data_segment = 2; /* just to prevent buffer overflow */
     }
     if (opts.mode != FORCE) {
 	if (!opts.do_format) { /* verify only */
