@@ -17,8 +17,8 @@
 
  *
  * $RCSfile: ftformat.h,v $
- * $Revision: 1.5 $
- * $Date: 1999/03/17 11:25:42 $
+ * $Revision: 1.6 $
+ * $Date: 2001/08/11 00:08:58 $
  *
  *      This file contains the definitions for the user level floppy
  *      tape formatting stuff for the QIC-40/80/3010/3020 floppy-tape
@@ -58,19 +58,36 @@ typedef struct {
 
 /*  Unaligned access.
  */
-#define PUT2(ptr, off, val) \
+#if !WORDS_BIGENDIAN
+# define PUT2(ptr, off, val) \
 { u_int16_t _val = val; memcpy((u_int8_t *)ptr + off, &_val, 2); }
-#define PUT4(ptr, off, val) \
+# define PUT4(ptr, off, val) \
 { u_int32_t _val = val; memcpy((u_int8_t *)ptr + off, &_val, 4); }
-#define PUT8(ptr, off, val) \
+# define PUT8(ptr, off, val) \
 { u_int64_t _val = val; memcpy((u_int8_t *)ptr + off, &_val, 8); }
-#define GET2(ptr, off) \
+# define GET2(ptr, off) \
 ({ u_int16_t _res; memcpy(&_res, (u_int8_t *)ptr + off, 2); _res; })
-#define GET4(ptr, off) \
+# define GET4(ptr, off) \
 ({ u_int32_t _res; memcpy(&_res, (u_int8_t *)ptr + off, 4); _res; })
-#define GET8(ptr, off) \
+# define GET8(ptr, off) \
 ({ u_int64_t _res; memcpy(&_res, (u_int8_t *)ptr + off, 8); _res; })
-
+#else
+# if __GLIBC__ < 2 || __GLIBC_MINOR__ < 2 || !__GNUC__ || __GNUC__ < 2
+#  error Only LITTLEENDIAN byte order supported, sorry
+# endif
+# define PUT2(ptr, off, val) \
+{ u_int16_t _val = bswap_16(val); memcpy((u_int8_t *)ptr + off, &_val, 2); }
+# define PUT4(ptr, off, val) \
+{ u_int32_t _val = bswap_32(val); memcpy((u_int8_t *)ptr + off, &_val, 4); }
+# define PUT8(ptr, off, val) \
+{ u_int64_t _val = bswap_64(val); memcpy((u_int8_t *)ptr + off, &_val, 8); }
+# define GET2(ptr, off) \
+({ u_int16_t _res; memcpy(&_res, (u_int8_t *)ptr + off, 2); bswap_16(_res); })
+# define GET4(ptr, off) \
+({ u_int32_t _res; memcpy(&_res, (u_int8_t *)ptr + off, 4); bswap_32(_res); })
+# define GET8(ptr, off) \
+({ u_int64_t _res; memcpy(&_res, (u_int8_t *)ptr + off, 8); bswap_64(_res); })
+#endif
 /* count the number of bits set in mask
  */
 static inline int count_ones(unsigned long mask)
