@@ -21,9 +21,9 @@
  *     the QIC-40/80/3010/3020 floppy-tape driver "ftape" for Linux.
  */
 
-char src[] = "$RCSfile: libftvt.c,v $";
-char rev[] = "$Revision: 1.14 $";
-char dat[] = "$Date: 2000/07/22 18:53:01 $";
+static char src[] = "$RCSfile: libftvt.c,v $";
+static char rev[] = "$Revision: 1.17 $";
+static char dat[] = "$Date: 2001/08/11 00:16:32 $";
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -43,8 +43,14 @@ char dat[] = "$Date: 2000/07/22 18:53:01 $";
 #include <ctype.h>
 #include <stdarg.h>
 #include <time.h>
+
+/* These are not in glibc 2.0, it seems, and are not required at the moment
+
 #include <iconv.h>
 #include <wchar.h>
+
+*/
+
 #include <libintl.h>
 #define _(String) gettext (String)
 
@@ -525,7 +531,8 @@ int ftvt_read_header_segment(int tape_fd, u_int8_t *hseg)
 	if (verbose) {
 		printf(_("done.\n"));
 	}
-	return (int)hseg[FT_FMT_CODE];
+	fmt_code = (int)hseg[FT_FMT_CODE];
+	return fmt_code;
 }
 
 /* Print the volume table.
@@ -636,7 +643,6 @@ static void print_char_line(FILE *where,
 /* print a volume table entry with signature "VTBL" */
 static void print_one_vtbl(const ftvt *volume)
 {
-	char sig[5];
 	char label[45];
 	int i;
 	const char *formats[] = { "Extended Format",
@@ -651,6 +657,8 @@ static void print_one_vtbl(const ftvt *volume)
 	printf("START %d\n", volume->start);
 	printf("END %d\n", volume->end);
 	printf("NUM_SEGMENTS %d\n", volume->num_segments);
+	memcpy(label, volume->ftvt_desc, 44);
+	label[44] = '\0';
 	printf("DESCRIPTION \"%s\"\n", label);
 	printf("DATE \"%s\"\n", ftvt_decode_date(volume->ftvt_date));
 	if (!volume->entry.vendor_specific ||
@@ -852,8 +860,6 @@ void ftvt_print_one_tagged(const ftvt *volume)
 
 	memcpy(sig, volume->ftvt_sig, 4);
 	sig[4] = '\0';
-	memcpy(label, volume->ftvt_desc, 44);
-	label[44] = '\0';
 	printf("SIGNATURE \"%s\"\n", sig);
 	for (i = 0; i < NR_ITEMS(prt_handler); i++) {
 		if (memcmp(sig, prt_handler[i].sig, 4) == 0) {
@@ -908,20 +914,22 @@ static char *ftvt_gets(FILE *fp, char *buffer, int *len)
 
 /**************** parsing of tagged input data *******************************/
 
-/* Internal use. Get a single value.
+/* Internal use. Get a single value and do something
  *
- * tag : the keyword
- * conv: conversion characater (like d, u, L etc.) for the numeric value
- * dest: Location to store the value in.
+ * tag :   the keyword
+ * conv:   conversion characater (like d, u, L etc.) for the numeric value
+ * dest:   Location to store the value in.
+ * action: something. Must allow a semicolon at end.
  *
  */
-#define GET_SINGLE_VALUE(tag, conv, dest)						\
+#define GET_SINGLE_VALUE_ACTION(tag, conv, dest, action)		\
 {																\
 	typeof(dest) value; /* cope with bitfields */				\
 																\
 	if (strncmp(#tag, buffer, sizeof(#tag)-1) == 0) {			\
 		if (sscanf(buffer, #tag" %"#conv, &value) == 1) {		\
 			(dest) = value;										\
+			action;												\
 		} else {												\
 			ftvt_error(_("Corrupt volume input data: %s\n"),	\
 					   buffer);									\
@@ -931,6 +939,15 @@ static char *ftvt_gets(FILE *fp, char *buffer, int *len)
 	}															\
 }
 
+/* Internal use. Get a single value.
+ *
+ * tag : the keyword
+ * conv: conversion characater (like d, u, L etc.) for the numeric value
+ * dest: Location to store the value in.
+ *
+ */
+#define GET_SINGLE_VALUE(tag, conv, dest) \
+	GET_SINGLE_VALUE_ACTION(tag, conv, dest, /**/)
 
 /* Internal use only. Get a string value.
  *
@@ -974,13 +991,14 @@ static char *ftvt_gets(FILE *fp, char *buffer, int *len)
  *
  * Return value:  -1 on error, the number of volumes otherwise.
  */
-int ftvt_parse_tagged(ftvt *volumes)
+int ftvt_parse_tagged(ftvt *volumes, int fmt_code)
 {
 	char *buffer = xmalloc(1024);
 	char *new;
 	int len = 1024;
 	int in_volume = 0;
 	int non_signature_seen = 0;
+	int start_seen = 0, end_seen = 0, num_segments_seen = 0;
 	enum { vtbl = 0, xtbl = 1, utid = 2, exvt = 3 } vtbl_id;
 	int num;
 	ftvt *volume = NULL;
@@ -1015,7 +1033,15 @@ int ftvt_parse_tagged(ftvt *volumes)
 					ftvt_error(_("Corrupt volume input data: %s\n"), buffer);
 					return -1;
 				}
+				/* convenience for sloppy input data */
+				if (start_seen && end_seen && !num_segments_seen) {
+					volume->num_segments = volume->end - volume->start + 1;
+				}
+				if (start_seen && num_segments_seen && !end_seen) {
+					volume->end = volume->start + volume->num_segments - 1;
+				}
 				in_volume = 0;
+				start_seen = end_seen = num_segments_seen = 0;
 			} else if (sscanf(buffer, "ENTRY %d", &num) == 1) {
 				if (num == maxnum) {
 					if (++(maxnum) > max_volumes) {
@@ -1068,9 +1094,10 @@ int ftvt_parse_tagged(ftvt *volumes)
 		});
 		switch (vtbl_id) {
 		case vtbl:
-			GET_SINGLE_VALUE(START, i, volume->start);
-			GET_SINGLE_VALUE(END, i, volume->end);
-			GET_SINGLE_VALUE(NUM_SEGMENTS, i, volume->num_segments);
+			GET_SINGLE_VALUE_ACTION(START, i, volume->start, start_seen = 1);
+			GET_SINGLE_VALUE_ACTION(END, i, volume->end, end_seen = 1);
+			GET_SINGLE_VALUE_ACTION (NUM_SEGMENTS, i, volume->num_segments,
+									 num_segments_seen = 1);
 			GET_STRING_VALUE(DESCRIPTION, volume->ftvt_desc, 44,
 							 memset(volume->ftvt_desc + strlen(value), ' ',
 									44 - strlen(value)));
